#### [BREAKING] The second argument `parsers` passed to `parsers.parse` has been removed (#13268 by @fisker)

The plugin's `print` function signature changed from

```ts
function parse(text: string, parsers: object, options: object): AST;
```

to

```ts
function parse(text: string, options: object): Promise<AST> | AST;
```

The second argument `parsers` has been removed, if you still need other parser during parse process, you can:

1. Import it your self

   ```js
   import parserBabel from "prettier/parser-babel.js";

   const myCustomPlugin = {
     parsers: {
       "my-custom-parser": {
         async parse(text) {
           const ast = await parserBabel.parsers.babel.parse(text);
           ast.program.body[0].expression.callee.name = "_";
           return ast;
         },
         astFormat: "estree",
       },
     },
   };
   ```

1. Get the parser from `options` argument

   ```js
   function getParserFromOptions(options, parserName) {
     for (const { parsers } of options.plugins) {
       if (
         parsers &&
         Object.prototype.hasOwnProperty.call(parsers, parserName)
       ) {
         return parsers[parserName];
       }
     }
   }

   const myCustomPlugin = {
     parsers: {
       "my-custom-parser": {
         async parse(text, options) {
           const babelParser = getParserFromOptions(options, "babel");
           const ast = await babelParser.parse(text);
           ast.program.body[0].expression.callee.name = "_";
           return ast;
         },
         astFormat: "estree",
       },
     },
   };
   ```
